{% extends 'appearance/base.html' %}

{% load i18n %}
{% load static %}

{% load common_tags %}

{% block base_title %}{% trans 'Password reset' %}{% endblock %}

{% block project_name %}{% endblock %}

{% block content_plain %}
    <div class="row">
        <div class="col-xs-10 col-xs-offset-1 col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3 col-lg-4 col-lg-offset-4">
            <div class="alert alert-success" role="alert">{% trans 'Password reset email sent!' %}</div>
        </div>
    </div>
{% endblock content_plain %}
