# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-14 21:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mailer', '0003_auto_20170703_1535'),
    ]

    operations = [
        migrations.AlterField(
            model_name='usermailer',
            name='default',
            field=models.BooleanField(default=True, help_text='If default, this mailing profile will be pre-selected on the document mailing form.', verbose_name='Default'),
        ),
    ]
